@@ -214,7 +214,7 @@ class KafkaConsumer(Consumer):
     # when a caller tries to use that functionality.
     return self._commit(offsets=offsets, asynchronous=True)
 
-  def subscribe(self, topics=None, pattern=None, listener=None):
+  def subscribe(self, topics=None, pattern=None, listener=None, **kwargs):
     self._log.info("Executing Consumer subscribe with topics=%s, pattern=%s, listener=%s", str(topics), str(pattern), str(listener))
     if not topics is None:
       topics = copy.copy(topics)
@@ -226,9 +226,11 @@ class KafkaConsumer(Consumer):
       else:
         topics.append(pattern)
     if listener is not None:
-      return super().subscribe(topics, listener=listener)
+      # the lambda expressions pass tuples with topic, partition, and offset (instead of just topic and partition),
+      # but that is compatible with the intended API of ConsumerRebalanceListener (see https://github.com/dpkp/kafka-python/blob/f19e4238fb47ae2619f18731f0e0e9a3762cfa11/kafka/consumer/subscription_state.py)
+      return super().subscribe(topics, on_assign=lambda c,tps: listener.on_partitions_assigned(tps), on_revoke=lambda c,tps: listener.on_partitions_revoked(tps), **kwargs)
     else:
-      return super().subscribe(topics)
+      return super().subscribe(topics, **kwargs)
 
   def poll(self, timeout_ms=0, max_records=None):
     rvm = {}
@@ -258,7 +260,7 @@ class KafkaConsumer(Consumer):
     self._log.info("Executing Consumer assign_and_seek with partoffs=%s.", str(partoffs))
     tps = []
     for tpo in partoffs:
-      if (tpo.offset > 0):
+      if (tpo.offset > 0) or (tpo.offset == OFFSET_BEGINNING) or (tpo.offset == OFFSET_END):
         tps.append(TopicPartitionOffset(tpo.topic, tpo.partition, tpo.offset))
       else:
         tps.append(TopicPartitionOffset(tpo.topic, tpo.partition))
@@ -268,10 +270,12 @@ class KafkaConsumer(Consumer):
     self._log.debug("Executing Consumer seek.")
     return super().seek(TopicPartitionOffset(tp.topic, tp.partition, offset))
 
-  def seek_to_beginning(self, tps=None):
+  def seek_to_beginning(self, *tps):
     self._log.debug("Executing Consumer seek_to_beginning.")
-    if tps is None or len(tps) < 1:
+    if len(tps) < 1:
       tps = self.assignment()
+    elif len(tps) == 1 and not isinstance(tps[0], TopicPartition):
+      tps = tps[0] # confluent_kafka all TPs as a single passed list convention
     for tp in tps:
       try:
         self.seek(tp, OFFSET_BEGINNING)
@@ -280,8 +284,10 @@ class KafkaConsumer(Consumer):
 
   def seek_to_end(self, tps=None):
     self._log.debug("Executing Consumer seek_to_end.")
-    if tps is None or len(tps) < 1:
+    if len(tps) < 1:
       tps = self.assignment()
+    elif len(tps) == 1 and not isinstance(tps[0], TopicPartition):
+      tps = tps[0] # confluent_kafka all TPs as a single passed list convention
     for tp in tps:
       try:
         self.seek(tp, OFFSET_END)
